/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.filesystem.client;

import java.io.InputStream;

import com.ibm.team.filesystem.common.IFileContent;
import com.ibm.team.filesystem.common.IFileItemHandle;
import com.ibm.team.filesystem.common.internal.util.FileContentProperties;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.scm.client.content.AbstractDownloadHandler;
import com.ibm.team.scm.common.IVersionableHandle;
import com.ibm.team.scm.common.IVersionedContent;

/**
 * Accepts events regarding a download being managed by a 
 * {@link IFileContentManagerSession}.
 *  
 * Clients should extend this class to implement custom download handlers
 * 
 * @since 0.7
 */
public class FileDownloadHandler extends AbstractDownloadHandler<FileContentProperties> {

	/** Called when the download has been canceled.
	 */
	public void downloadCanceled(IFileItemHandle handle, IFileContent content) {
	}
	
    /** Called when the download has been completed. Note that the last call 
     * to downloadStreamAcquired(IVersionableHandle, IVersionedContet, K, InputStream)} will complete before 
     * this method is called. 
     */
    public void downloadCompleted(IFileItemHandle handle, IFileContent content)
            throws TeamRepositoryException {
    }
    
    /** Called when the download starts.
     */
    public void downloadStart(IFileItemHandle handle, IFileContent content)
            throws TeamRepositoryException {
    }
    
    /** Called when an input stream has been acquired for the download. 
     */
    public void downloadStreamAcquired(IFileItemHandle handle, IFileContent content,
            InputStream in) throws TeamRepositoryException {
    }
    
    /**
     * Called when the download fails.
     */
    public void downloadFailed(IFileItemHandle handle, IFileContent content,
            Exception e) {
    }
    

    
    
    @Override
	public final void downloadCanceled(IVersionableHandle handle, IVersionedContent content, FileContentProperties context) {
		downloadCanceled((IFileItemHandle)handle, (IFileContent)content);
	}

    @Override
	public final void downloadCompleted(IVersionableHandle handle, IVersionedContent content, FileContentProperties context)
			throws TeamRepositoryException {
		downloadCompleted((IFileItemHandle)handle, (IFileContent)content);
	}

    @Override
	public final void downloadStart(IVersionableHandle handle, IVersionedContent content, FileContentProperties context)
			throws TeamRepositoryException {
		downloadStart((IFileItemHandle)handle, (IFileContent)content);
	}

    @Override
	public final void downloadStreamAcquired(IVersionableHandle handle, IVersionedContent content, FileContentProperties context,
			InputStream in) throws TeamRepositoryException {
		downloadStreamAcquired((IFileItemHandle)handle, (IFileContent)content, in);
	}

    @Override
	public final void downloadFailed(IVersionableHandle handle, IVersionedContent content, FileContentProperties context,
			Exception e) {
		downloadFailed((IFileItemHandle)handle, (IFileContent)content, e);
	}
}
