/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client;

import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Exception indicating a failure involving the file system.

 * @since 3.0
 * @SubclassNO Should not be subclassed above the Client API.
 */

@SuppressWarnings("serial") 
public class FileSystemException extends TeamRepositoryException {

    /**
     * Creates a new file system exception with the given message. The cause is
     * not initialized, and may subsequently be initialized by a call to
     * {@link Throwable#initCause}. The origin is initially <code>null</code>,
     * and may subsequently be initialized by a call to
     * {@link #setOrigin(Object)}.
     * 
     * @param message
     *            the detail message
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public FileSystemException(String message) {
        super(message);
    }

    /**
     * Creates a new file system exception for the given repository, with the
     * given message and optional underlying cause. The origin is initially
     * <code>null</code>, and may subsequently be initialized by a call to
     * {@link #setOrigin(Object)}.
     * <p>
     * Note that the detail message associated with <code>cause</code> is
     * <i>not </i> automatically incorporated in this exception's detail
     * message.
     * 
     * @param message
     *            the detail message
     * @param cause
     *            the underlying cause, which is saved for later retrieval by
     *            the {@link #getCause()}method); <code>null</code> indicates
     *            that the underlying cause is nonexistent or unknown
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public FileSystemException(String message, Throwable cause) {
        super(message, cause);
    }

    /**
     * Creates a new file system exception with the given origin and
     * message. The cause is not initialized, and may subsequently be
     * initialized by a call to {@link Throwable#initCause}.
     * 
     * @param origin
     *            the origin
     * @param message
     *            the detail message; must not be <code>null</code>
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public FileSystemException(Object origin, String message) {
        super(message);
        setOrigin(origin);
    }

    /**
     * Creates a new file system exception for the given underlying cause.
     * The detail message is that of the underlying cause, or the string
     * "Internal error" if it does not have one. This constructor is useful for
     * exceptions that are little more than wrappers for other throwables.
     * 
     * @param cause
     *            the underlying cause, which is saved for later retrieval by
     *            the {@link #getCause()}method); must not be <code>null</code>
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public FileSystemException(Throwable cause) {
        this(FileSystemException.getErrorMessage(cause), cause);
    }

    /**
     * Creates a new file system exception for the given origin, message,
     * and optional underlying cause.
     * <p>
     * Note that the detail message associated with <code>cause</code> is
     * <i>not </i> automatically incorporated in this exception's detail
     * message.
     * 
     * @param origin
     *            the origin
     * @param message
     *            the detail message
     * @param cause
     *            the underlying cause, which is saved for later retrieval by
     *            the {@link #getCause()}method); <code>null</code> indicates
     *            that the underlying cause is nonexistent or unknown
     * @ShortOp This is a short operation; it may block only momentarily; safe
     *          to call from a responsive thread.
     */
    public FileSystemException(Object origin, String message, Throwable cause) {
        this(message, cause);
        setOrigin(origin);
    }
    

    private static String getErrorMessage(Throwable cause) {
        if (cause == null)
            return null;
        String message = cause.getMessage();
        if (message == null) {
            message = "Internal error"; //$NON-NLS-1$
        }
        return message;
    }

}
