/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.filesystem.client;

import com.ibm.team.filesystem.client.internal.content.FileContentManager;
import com.ibm.team.filesystem.common.FileLineDelimiter;
import com.ibm.team.filesystem.common.IFileContent;
import com.ibm.team.filesystem.common.internal.util.FileContentProperties;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.scm.client.content.AbstractUploadHandler;
import com.ibm.team.scm.client.content.IVersionedContentManagerSession;
import com.ibm.team.scm.common.IVersionedContent;

/**
 * Accepts events regarding a upload being managed by an
 * {@link IVersionedContentManagerSession} and provides data needed
 * to upload content.
 * 
 * Clients should extend this class to implement custom upload handlers
 * @since 0.7
 */
public abstract class FileUploadHandler extends AbstractUploadHandler<FileContentProperties> {
    /**
     * Called to obtain the encoding
     */
    protected abstract String getEncoding() throws TeamRepositoryException;
    
    /**
     * Called to obtain the content type
     */
    protected abstract String getContentType() throws TeamRepositoryException;
    
    /**
     * Called to obtain the line delimiter
     */
    protected abstract FileLineDelimiter getLineDelimiter() throws TeamRepositoryException;
    
    /** Called when the upload has been completed.
     */
    protected void uploadCompleted(IFileContent content) throws TeamRepositoryException {
        
    }
    
    @Override
    public final void uploadCompleted(IVersionedContent content) throws TeamRepositoryException {
        uploadCompleted((IFileContent)content);
    }
    
	@Override
	public final FileContentProperties getContext() throws TeamRepositoryException {
		FileContentProperties context = new FileContentProperties(getEncoding(), getLineDelimiter());
		
		FileContentManager.validateArgumentsForUpload(getContentType(), context);
		
		return context;
	}
}
