/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2012. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client;

import org.eclipse.core.runtime.IProgressMonitor;


/**
 * Examines file content to determine MIME type, encoding of text files, 
 * line delimiters, etc. 
 * 
 * WE SHOULD always take an IShareable because the determination of encoding can 
 * be influenced by the properties of the IShareable.
 * 
 * @noimplement Should only be implemented internally.
 */
public interface IContentExaminer {
	
	/**
	 * Examine the content of a shareable to determine its type. The shareable 
	 * must exist on disk. If the file exists locally, this method should be 
	 * used instead of {@link #examine(String, IStreamSource, IProgressMonitor)}.
	 * 
	 * @param shareable The shareable to examine. 
	 * @param mon
	 * 
	 * @return The properties associated with the given shareable. Never <code>null</code>
	 * 
	 * @throws FileSystemException
	 * 
     * @longop This method should not be called from a responsive thread, 
     *      as it may block indefinitely.
     *
	 */
	public IContentProperties examine(IShareable shareable, IProgressMonitor mon) throws FileSystemException;
	
	/**
	 * Examine the given shareable and determine its content properties. The 
	 * values stored by jazz will be returned if they are present. If they are 
	 * not, then the content and name will be examined. 
	 * 
	 * @return The properties of the given shareable. Never <code>null</code>.
	 * 
     * @longop This method should not be called from a responsive thread, 
     *      as it may block indefinitely.
	 */
	public IContentProperties findStoredProperties(IShareable shareable, IProgressMonitor mon) throws FileSystemException;
	
	/**
	 * Examine the given shareable and determine its encoding.  If you are interested
	 * in the other properties such as Content type, etc.  Use {@link #findStoredProperties(IShareable, IProgressMonitor)}
	 * or {@link #examine(IShareable, IProgressMonitor)} instead.
	 * 
	 * @param shareable
	 * @param progress
	 * @return The encoding.  Never <code>null</code>.  All supported avenues will
	 * be exhausted in trying to determine the encoding.
	 */
	public String getEncoding(IShareable shareable, IProgressMonitor progress) throws FileSystemException;
}
