/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2014. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client;

import com.ibm.team.filesystem.common.FileLineDelimiter;
import com.ibm.team.filesystem.common.IFileContent;

/**
 * Encapsulates the properties of a file's content, eg: MIME type, encoding, 
 * line delimiter, etc. 
 * 
 * @noimplement Clients should not implement this interface. 
 */
public interface IContentProperties {

    public static final String CHARACTER_ENCODING_UNKNOWN = new String(""); //$NON-NLS-1$

    /**
     * Return the MIME type of the associated blob of bytes. 
     * It may be {@link IFileContent#CONTENT_TYPE_UNKNOWN}
     * if not known.
     */
    public String getMimeType();
    
    /** Return the encoding of a text file. If the file is not text, or the 
     * encoding is unknown, {@link #CHARACTER_ENCODING_UNKNOWN} is returned.
     * A <code>null</code> is never returned.
     */
    public String getEncoding();
    
    /**
     * Get the line delimiter used in the file. Never null.
     * May be {@link FileLineDelimiter#LINE_DELIMITER_NONE} if the file
     * is not text.
     */
    public FileLineDelimiter getLineDelimiter();
    
    /**
     * Get the content type for the type of merger that can do the storage merge.
     * @return The content type for merge purposes.  It may be null if it may be
     * binary data (i.e. no content type applicable for merging).
     */
    public String getContentTypeForMerge();
    
    /**
     * If the file maps to a text content (decided either via an Eclipse content type or
     * from the mime type).
     */
    public boolean isText();
    
}
