/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client;

import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.Collection;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.filesystem.client.internal.load.LoadRuleFactory;
import com.ibm.team.filesystem.client.operations.ILoadRule2;
import com.ibm.team.filesystem.common.IFileItemHandle;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.scm.client.IWorkspaceConnection;
import com.ibm.team.scm.common.IComponentHandle;
import com.ibm.team.scm.common.IVersionableHandle;

/**
 * Describe items to be loaded for a particular component within a workspace
 * 
 * This is still work in the progress.  The interface may change.
 * @deprecated use {@link ILoadRule2}
 */
public interface ILoadRule {

    /**
     * A factory which can be used to construct load rules
     */
    public static final ILoadRuleFactory loadRuleFactory = LoadRuleFactory.getInstance();
    
    /**
     * @return The component that the items will be loaded from
     */
    public IComponentHandle getComponent();
     
    /**
     * @return The workspace connection that the items will be loaded from
     */
    public IWorkspaceConnection getWorkspaceConnection();

    /**
     * Include the contents of the file supplied as a load rule
     * The file must follow a specific format to describe what should be loaded.
     * 
     * @param loadRules The file item within the component that describes what should
     * be loaded.
     * @param progress A monitor to show progress on.
     * @throws TeamRepositoryException If anything goes wrong.
     */
    public void addLoadRules(IFileItemHandle loadRules, IProgressMonitor progress) throws TeamRepositoryException;

    /**
     * Add the load rules specified in the given stream. Unless otherwise 
     * stated, no argument may be null.
     * 
     * @param stream A stream containing the contents of a loadrule file.
     * @param characterEncoding A platform encoding. It should be suitable for 
     *      passing into {@link Charset#forName(String)}.
     * @param progress A monitor to show progress. May be null. 
     */
    public void addLoadRules(InputStream stream, String characterEncoding, IProgressMonitor progress) throws TeamRepositoryException;
    
    /**
     * Based on the information within the load rule, provide the items that should be loaded.
     * The versionables have not been validated with respect to existence within the component, nor 
     * have they been validated for overlap.
     * 
     * @param progress Monitor to show progress on.
     * @return The set of folders that are to be loaded.  Never <code>null</code>.
     * @throws TeamRepositoryException
     */
    public Collection<IVersionableHandle> getItemsToLoad(IProgressMonitor progress) throws TeamRepositoryException;

}
