/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client;

import com.ibm.team.scm.common.IComponentHandle;
import com.ibm.team.scm.common.IContextHandle;


/**
 * Describes a set of local changes that are related tothe same workspace/component.
 * <p>
 * <b>Note:</b> This class/interface is part of an interim API that is still under development and expected to
 * change significantly before reaching stability. It is being made available at this early stage to solicit feedback
 * from pioneering adopters on the understanding that any code that uses this API will almost certainly be broken
 * (repeatedly) as the API evolves.
 * </p>
 * 
 * @since 0.6
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface ILocalChangeBatch {
    /**
     * Returns an array containing all local changes included in this batch.
     * 
     * @return
     */
    public ILocalChange[] getBatchedChanges();
    
    /**
     * Returns the connection for this batch of changes.
     * 
     * @return
     */
    public IContextHandle getConnection();
    
    /**
     * Returns the component for this batch of changes.
     * 
     * @return
     */
    public IComponentHandle getComponent();
}
