/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2015. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client;

import com.ibm.team.scm.common.IComponentHandle;
import com.ibm.team.scm.common.IContextHandle;

/**
 * Reports a group of local changes.
 * 
 * @since 0.6
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface ILocalChangeEvent {

    /**
     * Event kind constant indicating the changes in this event have occurred. 
     *
     * @see #getKind()
     */
    public int OCCURRED = 1;

    /**
     * Event kind constant indicating the changes in this event have been canceled. 
     *
     * @see #getKind()
     */
    public int CANCELED = 2;
    
    /**
     * Event kind constant indicating the changes in this event have been committed. 
     *
     * @see #getKind()
     */
    public int CONFIRMED = 3;

    /**
     * Returns all changes for the given workspace/component and kind of event.
     *    
     * @param connection the connection to look for changes for
     * @param component the component to look for changes for
     * @param sharePath share path to look for changes in
     * @param kind the kind of changes to retrieve
     * @see #OCCURRED
     * @see #CANCELED
     * @see #CONFIRMED
     * @return an array of changes
     */
    public ILocalChange[] getChanges(IContextHandle connection, IComponentHandle component, ILocation sharePath, int kind);

    /**
     * Returns whether this event has changes for the given workspace/component and kind of event.
     * @param connection the connection to look for changes for
     * @param component the component to look for changes for
     * @param kind the kind of changes to retrieve
     * @see #OCCURRED
     * @see #CANCELED
     * @see #CONFIRMED
     * @return whether this event has changes for the given workspace/component and kind of event
     */
    public boolean hasChanges(IContextHandle connection, IComponentHandle component, int kind);
    
    /**
     * Returns all batches of changes reported in this event for the given kind of event.
     * 
     * @param kind the kind of changes to retrieve
     * @see #OCCURRED
     * @see #CANCELED
     * @see #CONFIRMED 
     * @return all batches of changes for the given kind of event
     */
    public ILocalChangeBatch[] getBatches(int kind);

    /**
     * Returns whether this event has changes for the given workspace/component and kind of event.
     * @param connection the connection to look for changes for
     * @param component the component to look for changes for
     * @param cfaRoot the copy file area/sandbox directory
     * @param kind the kind of changes to retrieve
     * @see #OCCURRED
     * @see #CANCELED
     * @see #CONFIRMED
     * @return whether this event has changes for the given workspace/component and kind of event
     */
    public boolean hasChanges(IContextHandle connection, IComponentHandle component, ILocation cfaRoot, int kind);
}
