/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client;

import java.util.Collection;

import com.ibm.team.filesystem.client.operations.ICheckinOptions;
import com.ibm.team.filesystem.client.operations.IFileSystemOperation;
import com.ibm.team.filesystem.client.operations.IRefreshingOperation;
import com.ibm.team.filesystem.common.ILogicalChange;
import com.ibm.team.filesystem.common.ILogicalConflictReport;
import com.ibm.team.scm.client.IWorkspaceConnection;

/**
 * An operation that can be used to mark conflicts (or parts of conflicts as
 * having been merged.  Can be used for both resolve as merged (I changed
 * the stuff) and resolve with mine (mine is right ignore accepted) by
 * configuring the dilemma handler.
 * 
 * @since 3.0
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface IMarkAsMergedOperation extends IFileSystemOperation, IRefreshingOperation, ICheckinOptions {
    
    /**
     * Set the context of the operation.  That is, what workspace the conflicts
     * are to be resolved by applying the accepted/proposed changes.
     * 
     * @param connection The workspace containing the conflicts to be resolved.
     * Never <code>null</code>
     * 
     * @param conflictReport The conflict report describing all the conflicts in the
     * workspace. Never <code>null</code>
     */
    public void setContext(IWorkspaceConnection connection, ILogicalConflictReport conflictReport);

    /**
     * A group of changes have been specified to be resolved.  The changes
     * may depend upon other changes to also be resolved before they can be
     * resolved.  This will provide the changes that must also be resolved.
     * @return a collection of logical changes which have not been specified
     * to be resolved, but must be resolved in order for the changes so
     * far specified to be able to be resolved.
     */
    public Collection<ILogicalChange> getRequiredChanges();

    /**
     * Add a collection of changes to be applied.  It is possible
     * to configure the operation with a set of changes to resolve
     * with the proposed (accepted). Then discover additional changes
     * must also be applied and add those additional changes.
     * 
    * @param changes  The logical changes to be applied and the conflicts to be
    * resolved by overriding the current configuration with the unapplied change.  The
    * changes/conflicts must have been retrieved from the conflict report supplied
    * in {@link #setContext(IWorkspaceConnection, ILogicalConflictReport)}
    * Not <code>null</code>
     */
    public void addChangesToResolve(Collection<? extends ILogicalChange> moreChanges);

}
