/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2015. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client;

import java.util.Collection;

/**
 * An operation to remove a collection of local conflict descriptors from
 * the local change tracker.
 * 
 * This operation can be used to implement a resolve with mine action, removing
 * the local conflicts while preserving the local changes themselves.
 */
import com.ibm.team.filesystem.client.operations.IFileSystemOperation;

public interface IResolveLocalConflictsAsMergedOperation extends IFileSystemOperation {

    /**
     * Specify the conflicts to resolve
     * 
     * @param conflicts the conflicts to resolve. Must not be <code>null</code>
     */
    void addConflictsToResolve(Collection<ILocalConflict> conflicts);
    
    /**
     * Specify a conflict to resolve
     * 
     * @param conflics the conflict to resolve. Must not be <code>null</code>
     */
    void addConflictToResolve(ILocalConflict conflict);

}
