/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client;

import org.eclipse.core.runtime.IProgressMonitor;

/**
 * Describes a mapping between a local resource and a remote location. Currently
 * only projects can be shared.
 * 
 * @NoImplement This interface is not intended to be implemented by clients.
 * @since 0.6
 */
public interface IShare {
    /**
     * @return the sharing descriptor associated with this share,
     * never <code>null</code>.
     */
    public ISharingDescriptor getSharingDescriptor();

    /**
     * Returns the sandbox for this share's copy file area.
     *  
     * @return this share's sandbox, never <code>null</code>.
     * @since 0.7
     */
    public ISandbox getSandbox();

    /**
     * @return the path of the root shareable object for this share,
     * never <code>null</code>.  DO NOT assume there is only 1 segment
     * in the path.
     */
    public IRelativeLocation getPath();

    /**
     * @return the shareable for this share.  Never <code>null</code>
     */
    public IShareable getShareable();
    
    /**
     * Returns whether this share was loaded under an alternative name
     * or not.
     * @throws FileSystemException 
     */
    public boolean isLoadedWithAnotherName(IProgressMonitor monitor) throws FileSystemException;
}
