/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2012. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client;

import com.ibm.team.filesystem.common.FileLineDelimiter;
import com.ibm.team.scm.common.IComponentHandle;
import com.ibm.team.scm.common.IContextHandle;
import com.ibm.team.scm.common.IVersionable;
import com.ibm.team.scm.common.IVersionableHandle;

/**
 * Describes the state of an {@link IShareable} before any local changes. In 
 * other words, if a {@link IVersionable} is modified in the sandbox, this 
 * will return information about it from before the local change.
 * 
 * @NoImplement This interface should not be implemented by consumers. 
 */
public interface IUnmodifiedInfo {

    /** Return the original name of the item. */
    public String getName();

    
    /** Return the state of the executable bit. */
    public boolean getExecutable();
    
    /** Return the content type. */
    public String getContentType();
    
    /** Return the line delimiter. */
    public FileLineDelimiter getLineDelimiter();

    
    /**
     * Get the sandbox that contains this version. 
     */
    public ISandbox getSandbox();
    
    /**
     * Get the context this item was found in. 
     */
    public IContextHandle getContext();
    
    /**
     * Get the component containing this item. 
     */
    public IComponentHandle getComponent();
    
    /** Get the versionable associated with this item. 
     */
    public IVersionableHandle getVersionable();
}
