/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.filesystem.client.operations.IDiscardOperation;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.scm.client.IWorkspaceConnection;
import com.ibm.team.scm.common.IChangeSetHandle;

/**
 * This interface is used through the extension point com.ibm.team.filesystem.client.workItemLinksRemover. This is used inside the
 * {@link IDiscardOperation} to remove links between the discarded change sets and the related work items.
 * 
 * <p>This is only called if {@link IDiscardOperation#setRemoveWorkItemLinks(boolean)} is called with <code>true</code>.</p>
 */
public interface IWorkItemsLinksRemover {
    
    /**
     * Needs to be implemented by all extensions of the com.ibm.team.filesystem.client.workItemLinksRemover extension points.
     * @param connection the workspace connection
     * @param changesetHandles the given change set handles
     * @param monitor the given progress monitor
     * @throws TeamRepositoryException in case an error occurs while remove the work item links 
     */
    void removeWorkItemLinks(IWorkspaceConnection connection, List<IChangeSetHandle> changesetHandles, IProgressMonitor monitor) throws TeamRepositoryException;
}
