/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client;

import com.ibm.team.filesystem.common.IFileItemHandle;
import com.ibm.team.filesystem.common.ISymbolicLinkHandle;
import com.ibm.team.scm.common.IFolderHandle;
import com.ibm.team.scm.common.IVersionableHandle;

/**
 * The type of resource a shareable might be.
 * @since 2.0.0.2
 * 
 * @remoteProjects This enum is referenced by the remote project integration. 
 *      Downstream teams may need to adopt changes made to it.
 */
public enum ResourceType {
        FILE,
        FOLDER,
        SYMBOLIC_LINK;
        
        /**
         * Given a handle returns the type of resource type that it might be.
         * @param versionable The handle of a resource.  Never <code>null</code>
         * @return The type of Shareable Resource for that versionable.  Never <code>null</code>
         */
        public static ResourceType getResourceType(IVersionableHandle versionable) {
            ResourceType hint;
            if (versionable instanceof IFileItemHandle) {
                hint = ResourceType.FILE;
            } else if (versionable instanceof IFolderHandle) {
                hint = ResourceType.FOLDER;
            } else if (versionable instanceof ISymbolicLinkHandle) {
                hint = ResourceType.SYMBOLIC_LINK;
            } else {
                throw new IllegalArgumentException("Unexpected versionable type"); //$NON-NLS-1$
            }
            return hint;

        }
}
