/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.filesystem.client.daemon;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.Charset;

import com.ibm.team.filesystem.client.internal.http.HttpRequest;
import com.ibm.team.filesystem.client.internal.http.HttpResponse;
import com.ibm.team.repository.common.json.JSONObject;

public class JSONHandlerResponseEvent extends JSONHandlerEvent {

	private final Charset payloadCharset;
	private final byte[] payload;
	private final int payloadSize;
	private final int payloadOffset;

	public JSONHandlerResponseEvent(HttpRequest req, HttpResponse resp, Charset payloadCharset, byte[] payload) {
		super(req, resp);
		this.payloadCharset = payloadCharset;
		this.payload = payload;
		this.payloadSize = payload != null ? payload.length : -1;
		this.payloadOffset = payload != null ? 0 : -1;
	}
	
	public JSONHandlerResponseEvent(HttpRequest req, HttpResponse resp, Charset payloadCharset, byte[] payload, int payloadOffset, int payloadSize) {
		super(req, resp);
		this.payloadCharset = payloadCharset;
		this.payload = payload;
		this.payloadSize = payloadSize;
		this.payloadOffset = payloadOffset;
	}

	public String getFormattedPayload() {
		if (payload == null || payloadSize < 1) {
			return "(null)";
		}
		
		Object parsed;
		try {
			parsed = JSONObject.parseAny(new InputStreamReader(new ByteArrayInputStream(payload, payloadOffset, payloadSize), payloadCharset));
		} catch (IOException e) {
			StringBuffer buf = new StringBuffer("Exception decoding: " + e.getMessage() + "\nFull text: ");
			buf.append(new String(payload));
			return buf.toString();
		}
		
		if (parsed == null) {
			return "(null)";
		}
		
		return parsed.toString();
	}
}
