/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client.daemon;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.List;

import com.ibm.team.filesystem.client.internal.rest.IValidatingParameterWrapper;

public class JSONMethod {
	protected Object receiver;
	protected Method method;
	protected Class<? extends IValidatingParameterWrapper> parameterType;
	protected boolean takesProgress;
	protected Field [] parameterFields;
	protected Class<?> returnType;
	
	public JSONMethod(Object receiver, Method m) {
		this.receiver = receiver;
		this.method = m;
		Class<?> [] params = m.getParameterTypes();
		if (params.length == 1) {
			if (IValidatingParameterWrapper.class.isAssignableFrom(params[0])) {
				this.parameterType = params[0].asSubclass(IValidatingParameterWrapper.class);
			} else {
				takesProgress = true;
			}
		} else if (params.length == 2) {
			this.parameterType = params[0].asSubclass(IValidatingParameterWrapper.class);
			this.takesProgress = true;
		}
		initParameterFields();
		this.returnType = m.getReturnType();
	}
	
	protected void initParameterFields() {
		if (parameterType != null) {
			Field[] fld = parameterType.getDeclaredFields();
			List<Field> selected = new ArrayList<Field>(fld.length);
			for (Field f : fld) {
				int modifiers = f.getModifiers();
				
				if (!Modifier.isPublic(modifiers))
				    continue;
				if (Modifier.isTransient(modifiers))
				    continue;
				
				selected.add(f);
			}
			parameterFields = selected.toArray(new Field[selected.size()]);
		}
	}
	
	public Method getMethod() {
		return method;
	}
	
	public Object getReceiver() {
		return receiver;
	}
	
	public Class<? extends IValidatingParameterWrapper> getParameterType() {
		return parameterType;
	}
	
	public Class<?> getReturnType() {
		return returnType;
	}
	
	public Field[] getParameterFields() {
		return parameterFields;
	}
	
	public boolean takesProgress() {
		return takesProgress;
	}
}
