/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2013. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client.internal;

/**
 * Central listing of environment variables. 
 */
public abstract class EnvironmentVariables {

    /**
     * Environment variable allowing the user to prevent tracing.  
     */
    public static final String ENV_SCM_TRACE_DISABLE = "SCM_TRACE_DISABLE"; //$NON-NLS-1$

    /**
     * Environment variable indicating the path to the trace directory. 
     */
    public static final String ENV_SCM_TRACE_DIR = "SCM_TRACE_DIR"; //$NON-NLS-1$

    /**
     * Environment variable indicating whether or not rotated logs should be 
     * compressed. 
     */
    public static final String ENV_SCM_TRACE_COMPRESS = "SCM_TRACE_COMPRESS"; //$NON-NLS-1$

    /**
     * Environment variable specifying the maximum size of the trace files in 
     * the trace directory. 
     */
    public static final String ENV_SCM_TRACE_MAX_DIR_SIZE = "SCM_TRACE_MAX_DIR_SIZE"; //$NON-NLS-1$

    /**
     * Environemnt variable specifying the maximum size of traces files before 
     * they are rotated. 
     */
    public static final String ENV_SCM_TRACE_MAX_FILE_SIZE = "SCM_TRACE_MAX_FILE_SIZE"; //$NON-NLS-1$

    /**
     * Windows directory that holds app configuration that isn't sync'd with 
     * remote machines. 
     */
    public static final String LOCAL_APP_DATA = "LOCALAPPDATA"; //$NON-NLS-1$

    /** The name of the environment variable controlling how long a spawned 
     * daemon should wait for its initial connection. The value
     * must be an integer expressed in millis.
     */
    public static final String SCM_DAEMON_CONNECTION_TIME_OUT = "SCM_DAEMON_CONNECTION_TIME_OUT"; //$NON-NLS-1$

    /** The name of the environment variable controlling how long a spawned 
     * daemon should wait after all of its connections have closed. The value
     * must be an integer expressed in millis.
     */
    public static final String SCM_DAEMON_INACTIVE_TIME_OUT = "SCM_DAEMON_INACTIVE_TIME_OUT"; //$NON-NLS-1$

    public static final String SCM_DAEMON_START_DEBUG = "SCM_DAEMON_START_DEBUG"; //$NON-NLS-1$

    /**
     * Name of environment variable allowing the user to force a specific 
     * directory to be used for configuration. 
     */
    public static final String SCM_CONFIG_DIRECTORY = "SCM_CONFIG_DIRECTORY"; //$NON-NLS-1$
    
    /**
     * Name of environment variable that allows the CLI to run even if the 
     * directories containing sensitive information haven't been secured. 
     */
    public static final String SCM_ALLOW_INSECURE = "SCM_ALLOW_INSECURE"; //$NON-NLS-1$

    /**
     * Name of the environment variable that allows the user to configure 
     * the number of changes to be shown/interpreted in a change set.
     */
    public static final String SCM_MAX_CHANGES = "SCM_MAX_CHANGES"; //$NON-NLS-1$
    
    /**
     * Name of the environment variable that allows the user to configure 
     * the number of disk-backed hashmap entries for sharing metadata that
     * are stored in memory to optimize performance or memory usage.
     */
    public static final String SCM_MAX_METADATA_PAGING_SIZE = "SCM_MAX_METADATA_PAGING_SIZE"; //$NON-NLS-1$
    
    /**
     * Name of the environment variable that allow the user to configure
     * the maximum number of baselines displayed in an unfiltered baseline
     * query.  This is not a documented feature, and is being added for a
     * customer.
     */
    public static final String SCM_MAX_BASELINE_QUERY_RESULTS = "SCM_MAX_BASELINE_QUERY_RESULTS"; //$NON-NLS-1$

}
