/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client.internal;

import java.io.IOException;
import java.io.InputStream;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;

import com.ibm.team.filesystem.client.internal.utils.FileContentDilemmaHandler;
import com.ibm.team.filesystem.client.internal.utils.FileContentInputStream;
import com.ibm.team.filesystem.common.IFileContent;
import com.ibm.team.filesystem.common.IFileItemHandle;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.utils.IInputStreamProvider;

public class FileContentInputStreamProvider implements IInputStreamProvider {
	private ITeamRepository repo;
	private IFileItemHandle item;
	private IFileContent content;
    private FileContentDilemmaHandler contentDilemmaHandler;
	
    public FileContentInputStreamProvider(ITeamRepository repo, IFileItemHandle item, IFileContent content) {
        this(repo, item, content, null);
    }
    public FileContentInputStreamProvider(ITeamRepository repo, IFileItemHandle item, IFileContent content, FileContentDilemmaHandler contentDilemmaHandler) {
        super();
        if (!item.hasStateId()) {
            throw new IllegalArgumentException();
        }
        this.repo = repo;
        this.item = item;
        this.content = content;
        this.contentDilemmaHandler = contentDilemmaHandler == null ? FileContentDilemmaHandler.getDefault() : contentDilemmaHandler;
    }

    public InputStream getInputStream(IProgressMonitor monitor) throws IOException {
        if (monitor == null)
            monitor = new NullProgressMonitor();
        return new FileContentInputStream(repo, item, content, contentDilemmaHandler, monitor);
    }

    @Override
    public String toString() {
        return "FileContent(" + content.getHash() + ")"; //$NON-NLS-1$ //$NON-NLS-2$
    }
    
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((content == null) ? 0 : content.getHash().hashCode());
        result = prime * result + ((item == null) ? 0 : item.getItemId().hashCode());
        result = prime * result + ((item == null) ? 0 : item.getStateId().hashCode());
        result = prime * result + ((repo == null) ? 0 : repo.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final FileContentInputStreamProvider other = (FileContentInputStreamProvider) obj;
        if (content == null) {
            if (other.content != null)
                return false;
        } else if (!content.getHash().equals(other.content.getHash()))
            return false;
        if (item == null) {
            if (other.item != null) {
                return false;
            }
        } else if (!item.sameStateId(other.item)) {
            return false;
        }
        if (repo == null) {
            if (other.repo != null)
                return false;
        } else if (!repo.equals(other.repo))
            return false;
        return true;
    }

    public IFileContent getContent() {
        return content;
    }    
}
