/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client.internal;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;

import com.ibm.team.filesystem.client.FileSystemException;
import com.ibm.team.filesystem.client.ResourceType;
import com.ibm.team.repository.common.utils.IInputStreamProvider;

public class FileStorageInputStreamProvider implements IInputStreamProvider {

    private IFileStorage fileStorage;
    private IFileOptions options;
    
    private FileStorageInputStreamProvider(IFileOptions options, IFileStorage fileStorage) {
        super();
        this.options = options;
        this.fileStorage = fileStorage;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((fileStorage == null) ? 0 : fileStorage.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final FileStorageInputStreamProvider other = (FileStorageInputStreamProvider) obj;
        if (fileStorage == null) {
            if (other.fileStorage != null)
                return false;
        } else if (!fileStorage.equals(other.fileStorage))
            return false;
        return true;
    }

    public static FileStorageInputStreamProvider create(IFileOptions options, IFileStorage fileStorage) {
        return new FileStorageInputStreamProvider(options, fileStorage);
    }

    public static FileStorageInputStreamProvider createLink(IFileStorage fileStorage) {
        return new FileStorageInputStreamProvider(null, fileStorage);
    }

    public InputStream getInputStream(IProgressMonitor monitor) throws IOException {
        try {
            SubMonitor progress = SubMonitor.convert(monitor, 2);
            if (fileStorage.getResourceType(progress.newChild(1)) == ResourceType.SYMBOLIC_LINK) {
                String targetString = fileStorage.getLinkInfo(progress.newChild(1)).getTarget();
                return new ByteArrayInputStream(targetString.getBytes(Charset.defaultCharset().name()));
            }
            return fileStorage.getContents(options);
        } catch (FileSystemException e) {
            IOException e2 = new IOException();
            e2.initCause(e);
            throw e2;
        }
    }

}
