/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * (c) Copyright IBM Corporation 2005, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
package com.ibm.team.filesystem.client.workitems;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.filesystem.client.workitems.rest.parameters.ParmsPostChangeSetWorkItem;
import com.ibm.team.filesystem.client.workitems.rest.parameters.ParmsPostDeliverAndResolve;
import com.ibm.team.filesystem.client.workitems.rest.parameters.ParmsPostSubmitForReview;
import com.ibm.team.filesystem.client.workitems.rest.parameters.ParmsRefreshItems;
import com.ibm.team.filesystem.client.workitems.rest.parameters.ParmsWorkItemHierarchy;
import com.ibm.team.filesystem.common.workitems.internal.hierarchy.WorkItemHierarchyRootDTO;
import com.ibm.team.filesystem.common.workitems.internal.rest.DeliverAndResolveResultDTO;
import com.ibm.team.filesystem.common.workitems.internal.rest.SubmitForReviewResultDTO;
import com.ibm.team.repository.common.TeamRepositoryException;

public interface IFileSystemWorkItemRestClient {
	/**
	 * Deliver then resolve the associated work items.
	 * @param monitor Progress monitor to mark progress on.  Rest clients do not supply it,
     * progress is reported through informational messages.
	 */
	public DeliverAndResolveResultDTO postDeliverAndResolve(ParmsPostDeliverAndResolve parms, IProgressMonitor monitor) throws TeamRepositoryException;
	
	/**
	 * Submits work items for review that are associated with a change set. 
	 * @param monitor Progress monitor to mark progress on.  Rest clients do not supply it,
	 * progress is reported through informational messages.
	 */
	public SubmitForReviewResultDTO postSubmitForReview(ParmsPostSubmitForReview parms, IProgressMonitor monitor) throws TeamRepositoryException;
	
	/**
	 * Add a work item to a change set. If the work item is already assigned, the operation will 
	 * be a noop but no exception will be thrown.
	 * @param monitor Progress monitor to mark progress on.  Rest clients do not supply it,
     * progress is reported through informational messages.
	 */
    public void postAddWorkItem(ParmsPostChangeSetWorkItem parms, IProgressMonitor monitor) throws TeamRepositoryException;

    /**
     * Remove a work item from a change set.
     * @param monitor Progress monitor to mark progress on.  Rest clients do not supply it,
     * progress is reported through informational messages.
     */
    public void postRemoveWorkItem(ParmsPostChangeSetWorkItem parms, IProgressMonitor monitor) throws TeamRepositoryException;
    
    /**
     * Returns the root of a hierarchy of work items and change sets.
     */
    public WorkItemHierarchyRootDTO getWorkItemHierarchy(ParmsWorkItemHierarchy parms, IProgressMonitor monitor) throws TeamRepositoryException;
    
    /**
     * refresh the given items in the client cache. This is needed since the sync model references work items
     * but does not refetch them when the model is refreshed.
     * 
     * @param parms the parms
     * @param monitor a progress monitor
     * @throws TeamRepositoryException
     */
    public void postRefreshWorkItems(ParmsRefreshItems parms, IProgressMonitor monitor)  throws TeamRepositoryException;

}
