/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client.workitems;

import com.ibm.team.filesystem.client.operations.DeliverDilemmaHandler;
import com.ibm.team.filesystem.client.workitems.internal.WorkItemOperationFactory;
import com.ibm.team.filesystem.client.workitems.operations.DeliverAndFixDilemmaHandler;
import com.ibm.team.filesystem.client.workitems.operations.ICheckinAndDeliverOperation;
import com.ibm.team.filesystem.client.workitems.operations.IDeliverAndFixOperation;
import com.ibm.team.filesystem.client.workitems.operations.ISubmitForReviewOperation;
import com.ibm.team.filesystem.client.workitems.operations.SubmitForReviewDilemmaHandler;

public interface IWorkItemOperationFactory {
    /**
     * Operation factory to be used to create new operations that can 
     * be configured and run against workspaces and copy file areas. These operations
     * deal with work items.
     */
    public static final IWorkItemOperationFactory instance = WorkItemOperationFactory.getInstance();

	/**
	 * Provides an operation that can be used to deliver changes and components
	 * to a target workspace, associate a work item to the change set, and
	 * resolve the work item.
	 * 
	 * @param dilemmaHandler
	 *            The dilemmaHandler that will be called if problems are
	 *            encountered during the deliver. Can be <code>null</code> in
	 *            which case a default handler will be used.
	 * @return A deliver and resolve operation to be used to deliver changes and
	 *         components to a start workspace and resolve the work item.
	 */
	public IDeliverAndFixOperation getDeliverAndResolveOperation(DeliverAndFixDilemmaHandler dilemmaHandler);
	
	/**
	 * Provides an operation that can be used to associate a work item to change sets
	 * and create an approval for linked work items. The operation can optionally
	 * suspend the change sets.
	 * 
	 * @param dilemmaHandler
	 * 			The dilemmaHandler that will be called if problems are encountered
	 * 			during the operation. Can be <code>null</code> in which case a
	 * 			default handler will be used.
	 * @return A submit for review operation to be used to submit change sets for
	 * 			review with a linked work item.
	 */
	public ISubmitForReviewOperation getSubmitForReviewOperation(SubmitForReviewDilemmaHandler dilemmaHandler);
	
	/**
	 * Provides an operation that can be used to checkin local changes into a
	 * changeset and have the change sets delivered.  The operation can also 
	 * link the change set with a workitem and set the comment on it.
	 * 
	 * @param dilemmaHandler
	 * 			The dilemmaHandler that will be called if problems are encountered
	 * 			during the operation. Can be <code>null</code> in which case a
	 * 			default handler will be used.
	 * @return A checkin and deliver operation to be used to check in changes,
	 * 			link them with a work item and deliver it
	 */
	public ICheckinAndDeliverOperation getCheckinAndDeliverOperation(DeliverDilemmaHandler dilemmaHandler);
}
