/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
package com.ibm.team.filesystem.client.workitems.internal;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.filesystem.client.workitems.IFileSystemWorkItemManager;
import com.ibm.team.filesystem.common.workitems.ILinkConstants;
import com.ibm.team.links.common.ILink;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.scm.common.IChangeSetHandle;
import com.ibm.team.scm.common.links.IComponentLinkCreator;
import com.ibm.team.workitem.common.model.IWorkItemHandle;

public class ChangeSetLinkComponentCreator implements IComponentLinkCreator {

	public ILink create(String linkTypeId, Object source, Object target,
			IProgressMonitor monitor) throws TeamRepositoryException {
		if(linkTypeId.equals(ILinkConstants.CHANGESET_WORKITEM_LINKTYPE_ID) && source instanceof IChangeSetHandle && target instanceof IWorkItemHandle) {
			IChangeSetHandle changeSetHandle = (IChangeSetHandle)source;
			IWorkItemHandle workItemHandle = (IWorkItemHandle) target;
			ITeamRepository repo = (ITeamRepository) changeSetHandle.getOrigin();
			if(repo == null) {
				repo = (ITeamRepository)workItemHandle.getOrigin();
			}
			if(repo == null)
				throw new IllegalStateException("Missing origin on both change set handle and work item handle"); //$NON-NLS-1$
			IFileSystemWorkItemManager workItemManager = (IFileSystemWorkItemManager) repo.getClientLibrary(IFileSystemWorkItemManager.class);
			List<ILink> links = workItemManager.createLink(null, changeSetHandle, new IWorkItemHandle[] {workItemHandle}, monitor);
			return links.isEmpty() ? null : links.get(0);
		}
		return null;
	}

	public boolean handles(String linkTypeId) {
		return ILinkConstants.CHANGESET_WORKITEM_LINKTYPE_ID.equals(linkTypeId);
	}
}
