/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * (c) Copyright IBM Corporation 2005, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
package com.ibm.team.filesystem.client.workitems.internal;

import java.net.URI;
import java.net.URISyntaxException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.osgi.util.NLS;

import com.ibm.team.filesystem.client.internal.LoggingHelper;
import com.ibm.team.filesystem.client.internal.calm.OslcLinkUtil;
import com.ibm.team.filesystem.client.internal.rest.CommonUtil;
import com.ibm.team.filesystem.client.internal.rest.util.CommitUtil;
import com.ibm.team.filesystem.client.internal.rest.util.DeliverUtil;
import com.ibm.team.filesystem.client.internal.rest.util.RefreshUtil;
import com.ibm.team.filesystem.client.internal.rest.util.SuspendResumeDiscardChangeSetUtil;
import com.ibm.team.filesystem.client.operations.DeliverDilemmaHandler;
import com.ibm.team.filesystem.client.operations.IDilemmaHandler;
import com.ibm.team.filesystem.client.operations.SuspendDilemmaHandler;
import com.ibm.team.filesystem.client.rest.IFilesystemRestClient;
import com.ibm.team.filesystem.client.rest.parameters.ParmsChangeSet;
import com.ibm.team.filesystem.client.rest.parameters.ParmsDeliver;
import com.ibm.team.filesystem.client.rest.parameters.ParmsSuspendChangeSets;
import com.ibm.team.filesystem.client.workitems.IFileSystemWorkItemManager;
import com.ibm.team.filesystem.client.workitems.IFileSystemWorkItemRestClient;
import com.ibm.team.filesystem.client.workitems.IWorkItemOperationFactory;
import com.ibm.team.filesystem.client.workitems.internal.LocateChangeSetsUtil.LcsEditorChangeSetEntry;
import com.ibm.team.filesystem.client.workitems.internal.LocateChangeSetsUtil.LcsEditorChangeSetsResult;
import com.ibm.team.filesystem.client.workitems.internal.LocateChangeSetsUtil.LcsEditorEntry;
import com.ibm.team.filesystem.client.workitems.internal.LocateChangeSetsUtil.LcsEditorUnassignedWorkItemEntry;
import com.ibm.team.filesystem.client.workitems.internal.LocateChangeSetsUtil.LcsEditorWorkItemEntry;
import com.ibm.team.filesystem.client.workitems.internal.LocateChangeSetsUtil.ParmsGetLcsEditorChangeSets;
import com.ibm.team.filesystem.client.workitems.operations.DeliverAndFixDilemmaHandler;
import com.ibm.team.filesystem.client.workitems.operations.DeliverAndFixParameter;
import com.ibm.team.filesystem.client.workitems.operations.IDeliverAndFixOperation;
import com.ibm.team.filesystem.client.workitems.operations.ISubmitForReviewOperation;
import com.ibm.team.filesystem.client.workitems.operations.SubmitForReviewDilemmaHandler;
import com.ibm.team.filesystem.client.workitems.operations.SubmitForReviewParameter;
import com.ibm.team.filesystem.client.workitems.operations.SubmitForReviewParameter.ChangeSetReview;
import com.ibm.team.filesystem.client.workitems.rest.parameters.ParmsChangeSetsToSource;
import com.ibm.team.filesystem.client.workitems.rest.parameters.ParmsPostChangeSetWorkItem;
import com.ibm.team.filesystem.client.workitems.rest.parameters.ParmsPostDeliverAndResolve;
import com.ibm.team.filesystem.client.workitems.rest.parameters.ParmsPostSubmitForReview;
import com.ibm.team.filesystem.client.workitems.rest.parameters.ParmsRefreshItems;
import com.ibm.team.filesystem.client.workitems.rest.parameters.ParmsWorkItem;
import com.ibm.team.filesystem.client.workitems.rest.parameters.ParmsWorkItemHierarchy;
import com.ibm.team.filesystem.common.internal.rest.client.core.ChangeSetDTO;
import com.ibm.team.filesystem.common.internal.rest.client.core.ComponentDTO;
import com.ibm.team.filesystem.common.internal.rest.client.core.ContributorDTO;
import com.ibm.team.filesystem.common.internal.rest.client.core.FilesystemRestClientDTOcoreFactory;
import com.ibm.team.filesystem.common.workitems.internal.hierarchy.FilesystemWorkItemsRestDTOhierarchyFactory;
import com.ibm.team.filesystem.common.workitems.internal.hierarchy.WorkItemHierarchyNodeDTO;
import com.ibm.team.filesystem.common.workitems.internal.hierarchy.WorkItemHierarchyRootDTO;
import com.ibm.team.filesystem.common.workitems.internal.rest.DeliverAndResolveResultDTO;
import com.ibm.team.filesystem.common.workitems.internal.rest.FilesystemWorkItemsRestDTOFactory;
import com.ibm.team.filesystem.common.workitems.internal.rest.SubmitForReviewResultDTO;
import com.ibm.team.filesystem.common.workitems.internal.rest.WorkItemDTO;
import com.ibm.team.filesystem.common.workitems.internal.rest.WorkItemResultDTO;
import com.ibm.team.links.common.IItemReference;
import com.ibm.team.links.common.ILink;
import com.ibm.team.links.common.IReference;
import com.ibm.team.links.common.IURIReference;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.client.TeamPlatform;
import com.ibm.team.repository.common.IContributor;
import com.ibm.team.repository.common.IContributorHandle;
import com.ibm.team.repository.common.IItemHandle;
import com.ibm.team.repository.common.Location;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.UUID;
import com.ibm.team.scm.client.IWorkspaceConnection;
import com.ibm.team.scm.client.IWorkspaceManager;
import com.ibm.team.scm.client.SCMPlatform;
import com.ibm.team.scm.common.IChangeSet;
import com.ibm.team.scm.common.IChangeSetHandle;
import com.ibm.team.scm.common.IComponent;
import com.ibm.team.scm.common.IComponentHandle;
import com.ibm.team.scm.common.IWorkspace;
import com.ibm.team.scm.common.IWorkspaceHandle;
import com.ibm.team.scm.common.WorkspaceComparisonFlags;
import com.ibm.team.scm.common.dto.IChangeHistorySyncReport;
import com.ibm.team.scm.common.internal.dto.CreateLinksResult;
import com.ibm.team.scm.common.internal.util.RestUtils;
import com.ibm.team.scm.common.links.ChangeSetLinks;
import com.ibm.team.scm.common.links.ILinkConstants;
import com.ibm.team.scm.common.providers.ProviderFactory;
import com.ibm.team.workitem.client.IAuditableClient;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.IWorkItemHandle;

public class FileSystemWorkItemRestClient implements
		IFileSystemWorkItemRestClient {

    private static final UUID NULL_STATE = null;
    
	public void postAddWorkItem(ParmsPostChangeSetWorkItem parms, IProgressMonitor monitor)
			throws TeamRepositoryException {
			ITeamRepository repo = CommonUtil.getTeamRepository(parms.repositoryUrl);
			if(! repo.loggedIn())
				repo.login(null);

			ITeamRepository wirepo = null;
			if(RestUtils.isSet(parms.wiRepositoryUrl))
				wirepo = CommonUtil.getTeamRepository(parms.wiRepositoryUrl);
			
			IWorkspace workspace = null;
			IChangeSetHandle changeSetHandle = (IChangeSetHandle) IChangeSet.ITEM_TYPE.createItemHandle(repo, UUID.valueOf(parms.changeSetId), NULL_STATE);;
			IWorkItemHandle workItemHandle = (IWorkItemHandle) IWorkItem.ITEM_TYPE.createItemHandle(repo, UUID.valueOf(parms.workItemItemId), NULL_STATE);;

			if(RestUtils.isSet(parms.workspaceId)) {
				IWorkspaceHandle wsHandle = (IWorkspaceHandle) IWorkspace.ITEM_TYPE.createItemHandle(repo, UUID.valueOf(parms.workspaceId), NULL_STATE);
				IWorkspaceConnection connection = SCMPlatform.getWorkspaceManager(repo).getWorkspaceConnection(wsHandle, monitor);
				workspace = connection.getResolvedWorkspace();
			}

			if(!(wirepo == null || wirepo.getRepositoryURI().equals(repo.getRepositoryURI()))){	
				createCsCrLink(parms, repo, workspace, changeSetHandle, monitor);
			}else{
				IFileSystemWorkItemManager manager = (IFileSystemWorkItemManager) repo.getClientLibrary(IFileSystemWorkItemManager.class);
				manager.createLink(workspace, changeSetHandle, new IWorkItemHandle[] { workItemHandle }, monitor);
			}
	}
	
	public void postRemoveWorkItem(ParmsPostChangeSetWorkItem parms, IProgressMonitor monitor)
			throws TeamRepositoryException {
		ITeamRepository repo = CommonUtil.getTeamRepository(parms.repositoryUrl);
		if(! repo.loggedIn())
			repo.login(null);

		ITeamRepository wirepo = null;
		if(RestUtils.isSet(parms.wiRepositoryUrl))
			wirepo = CommonUtil.getTeamRepository(parms.wiRepositoryUrl);
		
		IChangeSetHandle changeSetHandle = (IChangeSetHandle) IChangeSet.ITEM_TYPE.createItemHandle(repo, UUID.valueOf(parms.changeSetId), NULL_STATE);;

		if(!(wirepo == null || wirepo.getRepositoryURI().equals(repo.getRepositoryURI()))){
			removeCsCrLink(parms, repo, changeSetHandle, monitor);
			return;
		}

		IWorkItemHandle workItemHandle = (IWorkItemHandle) IWorkItem.ITEM_TYPE.createItemHandle(repo, UUID.valueOf(parms.workItemItemId), NULL_STATE);;

		IFileSystemWorkItemManager manager = (IFileSystemWorkItemManager) repo.getClientLibrary(IFileSystemWorkItemManager.class);
		ProviderFactory providerFactory = (ProviderFactory) repo.getClientLibrary(ProviderFactory.class);
		List<ILink> links = ChangeSetLinks.findLinks(providerFactory, changeSetHandle, new String[] {ILinkConstants.CHANGESET_WORKITEM_LINKTYPE_ID }, monitor);
		ILink linkToRemove = null;

		for (ILink link : links) {
			IReference targetRef = link.getTargetRef();
			if(targetRef.isItemReference()) {
				IItemHandle refHandle = ((IItemReference)targetRef).getReferencedItem();
				if(refHandle instanceof IWorkItemHandle) {
					if(refHandle.sameItemId(workItemHandle)) {
						linkToRemove = link;
					}
				}
			}
		}

		manager.deleteLinks(Collections.singletonList(linkToRemove), monitor);

	}

	public DeliverAndResolveResultDTO postDeliverAndResolve(ParmsPostDeliverAndResolve parms, IProgressMonitor progress)
			throws TeamRepositoryException {
		SubMonitor monitor = SubMonitor.convert(progress, 100);
		
		ITeamRepository repo;
		IWorkspaceConnection source = parms.source.getWorkspaceConnection(monitor.newChild(10));
		IWorkspaceConnection target = parms.target.getWorkspaceConnection(monitor.newChild(10));
		Collection<IChangeSetHandle> changeSetHandles = new HashSet<IChangeSetHandle>();
		List<IWorkItemHandle> workItemHandles = new ArrayList<IWorkItemHandle>();
		Collection<IContributorHandle> reviewers = new HashSet<IContributorHandle>();
		boolean shouldResolve = parms.shouldResolve.booleanValue();
		boolean addReview = parms.reviewers != null;

		if (addReview) {
			repo = parms.reviewers.getTeamRepository();
			for (String reviewer : parms.reviewers.contributorItemIds) {
				reviewers.add((IContributorHandle) IContributor.ITEM_TYPE.createItemHandle(repo, UUID.valueOf(reviewer), null));
			}
		}
		
		repo = CommonUtil.getTeamRepository(parms.source.repositoryUrl);
		for (String changeSetId : parms.changeSets) {
			IChangeSetHandle changeSetHandle = (IChangeSetHandle) IChangeSet.ITEM_TYPE
					.createItemHandle(repo, UUID.valueOf(changeSetId), NULL_STATE);
			changeSetHandles.add(changeSetHandle);
		}
		
		SubMonitor iterMonitor = monitor.newChild(10);
		iterMonitor.setWorkRemaining(parms.workItemIds.length);
		for (ParmsWorkItem parmWorkItem : parms.workItemIds) {
			workItemHandles.add(parmWorkItem.getWorkItem(iterMonitor.newChild(1)));
		}
		
		IChangeHistorySyncReport syncReport = source.compareTo(target,
				WorkspaceComparisonFlags.INCLUDE_BASELINE_INFO,
				Collections.EMPTY_LIST, null);
		repo = TeamPlatform.getTeamRepositoryService().getTeamRepository(parms.source.repositoryUrl);
		IContributorHandle[] reviewList = new IContributorHandle[reviewers.size()];
		DeliverAndFixParameter opParms = new DeliverAndFixParameter(
				changeSetHandles, parms.workItemComment, parms.approvalSubject, 
				reviewers.toArray(reviewList), workItemHandles,
				syncReport, source, target, shouldResolve, parms.dueDate != null ? new Timestamp(parms.dueDate.longValue()) : null);
		
		ParmsDeliver deliverParms = new ParmsDeliver();
		deliverParms.sandboxUpdateDilemmaHandler = parms.sandboxUpdateDilemmaHandler;
		deliverParms.deliverDilemmaHandler = parms.deliverHandler;
		deliverParms.updateDilemmaHandler = parms.updateDilemmaHandler;
		deliverParms.pendingChangesDilemmaHandler = parms.pendingChangesHandler;
		
		final DeliverAndResolveResultDTO result = FilesystemWorkItemsRestDTOFactory.eINSTANCE.createDeliverAndResolveResultDTO();
		final DeliverDilemmaHandler deliverDilemmaHandler = DeliverUtil.getDeliverDilemmaHandler(deliverParms, result, monitor.newChild(10));
		
		// Default is to fail
		int wiAction = IDilemmaHandler.FAIL;
		if (parms.workItemHandler != null) {
			String action = parms.workItemHandler.unreachableWorkItemInstruction;
			if (IFilesystemRestClient.CONTINUE.equals(action)) {
				wiAction = IDilemmaHandler.CONTINUE;
			} else if (IFilesystemRestClient.CANCEL.equals(action)) {
				wiAction = IDilemmaHandler.CANCEL;
			}
		}
		
		final int unreachableWorkItemAction = wiAction;
		DeliverAndFixDilemmaHandler dilemmaHandler = new DeliverAndFixDilemmaHandler() {
			@Override
			public DeliverDilemmaHandler getDeliverDilemmaHandler() {
				return deliverDilemmaHandler;
			}
			
			@Override
			public int workItemsUnreachable(int numUnreachable, int total) {
				WorkItemResultDTO workItemResult = FilesystemWorkItemsRestDTOFactory.eINSTANCE.createWorkItemResultDTO();
				
				workItemResult.setNumberOfUnreachableWorkItems(numUnreachable);
				workItemResult.setTotalWorkItems(total);
				
				result.setWorkItemResult(workItemResult);
				
				return unreachableWorkItemAction;
			}
		};
		
		IDeliverAndFixOperation deliverAndFixOperation = IWorkItemOperationFactory.instance.getDeliverAndResolveOperation(dilemmaHandler);
		deliverAndFixOperation.deliverAndFix(opParms);
		RefreshUtil.configureRefresh(parms.preOpRefresh, deliverAndFixOperation);
		CommitUtil.configureCommit(parms.pendingChangesHandler, deliverAndFixOperation);
		try {
			deliverAndFixOperation.run(monitor.newChild(60));
		} catch (OperationCanceledException e) {
			// Details of the cancellation and the reasons will be set in the result
			result.setCancelled(true);
		}
		
		return result;
	}

	public SubmitForReviewResultDTO postSubmitForReview(final ParmsPostSubmitForReview parms,
			IProgressMonitor progress) throws TeamRepositoryException {
		SubMonitor monitor = SubMonitor.convert(progress, 100);
		
		ITeamRepository repo;
		Collection<IContributorHandle> reviewers = new HashSet<IContributorHandle>();
		Collection<IChangeSetHandle> changeSetHandles = new HashSet<IChangeSetHandle>();
		Collection<ChangeSetReview> changeSetReviews = new HashSet<ChangeSetReview>();
		List<IWorkItemHandle> workItemHandles = new ArrayList<IWorkItemHandle>();
		
		repo = parms.reviewers.getTeamRepository();
		for (String reviewer : parms.reviewers.contributorItemIds) {
			reviewers.add((IContributorHandle) IContributor.ITEM_TYPE.createItemHandle(repo, UUID.valueOf(reviewer), null));
		}
		
		for (ParmsChangeSetsToSource changeSetsToReview : parms.changeSetsToReview) {
			for (String changeSetId : changeSetsToReview.changeSetIds) {
				IChangeSetHandle changeSetHandle = (IChangeSetHandle) IChangeSet.ITEM_TYPE
						.createItemHandle(UUID.valueOf(changeSetId), NULL_STATE);
				IWorkspaceConnection source = changeSetsToReview.source.getWorkspaceConnection(monitor.newChild(5));
				ChangeSetReview changeSetReview = new ChangeSetReview(changeSetHandle, source);
				changeSetReviews.add(changeSetReview);
				changeSetHandles.add(changeSetHandle);
			}
		}
		
		for (ParmsWorkItem parmWorkItem : parms.workItemIds) {
			workItemHandles.add(parmWorkItem.getWorkItem(monitor.newChild(5)));
		}
		
		IContributorHandle[] reviewList = new IContributorHandle[reviewers.size()];
		final SubmitForReviewResultDTO result = FilesystemWorkItemsRestDTOFactory.eINSTANCE.createSubmitForReviewResultDTO();
		
		SubmitForReviewParameter opParms = new SubmitForReviewParameter(parms.workItemComment,
				reviewers.toArray(reviewList), parms.suspendChangeSets.booleanValue(), 
				parms.approvalSubject, workItemHandles, changeSetReviews, repo,
				parms.dueDate != null ? new Timestamp(parms.dueDate.longValue()) : null);
		
		ParmsSuspendChangeSets suspendParms = new ParmsSuspendChangeSets();
		suspendParms.sandboxUpdateDilemmaHandler = parms.sandboxUpdateDilemmaHandler;
		suspendParms.outOfSyncInstructions = parms.outOfSyncInstructions;
		suspendParms.pendingChangesDilemmaHandler = parms.pendingChangesHandler;
		final SuspendDilemmaHandler suspendDilemmaHandler = SuspendResumeDiscardChangeSetUtil
				.getSuspendDilemmaHandler(suspendParms,
						result.getOutOfSyncShares(), 
						result.getConfigurationsWithUncheckedInChanges(),
						result.getCommitDilemma(),
						result.getSandboxUpdateDilemma(),
						result.getUpdateDilemma(),
						result.getLocksToRelease(),
						monitor.newChild(5));
		
		// Default is to fail
		int wiAction = IDilemmaHandler.FAIL;
		if (parms.workItemHandler != null) {
			String action = parms.workItemHandler.unreachableWorkItemInstruction;
			if (IFilesystemRestClient.CONTINUE.equals(action)) {
				wiAction = IDilemmaHandler.CONTINUE;
			} else if (IFilesystemRestClient.CANCEL.equals(action)) {
				wiAction = IDilemmaHandler.CANCEL;
			}
		}
		
		final int unreachableWorkItemAction = wiAction;
		SubmitForReviewDilemmaHandler dilemmaHandler = new SubmitForReviewDilemmaHandler() {
			@Override
			public SuspendDilemmaHandler getSuspendDilemmaHandler() {
				return suspendDilemmaHandler;
			}
			
			@Override
			public int workItemsUnreachable(int numUnreachable, int total) {
				WorkItemResultDTO workItemResult = FilesystemWorkItemsRestDTOFactory.eINSTANCE.createWorkItemResultDTO();
				workItemResult.setNumberOfUnreachableWorkItems(numUnreachable);
				workItemResult.setTotalWorkItems(total);
				
				result.setWorkItemResult(workItemResult);
				return unreachableWorkItemAction;
			}
		};
		
		ISubmitForReviewOperation submitOp = IWorkItemOperationFactory.instance.getSubmitForReviewOperation(dilemmaHandler);
		submitOp.submitForReview(opParms);
		RefreshUtil.configureRefresh(parms.preOpRefresh, submitOp);
		CommitUtil.configureCommit(parms.pendingChangesHandler, submitOp);
		try {
			submitOp.run(monitor.newChild(85));
		} catch (OperationCanceledException e) {
			result.setCancelled(true);
		}
		
		return result;
	}

	// Use LocateChangeSetsUtil to query the server for a work item hierarchy.
	// Map the input parms and the output result.
	public WorkItemHierarchyRootDTO getWorkItemHierarchy(
			ParmsWorkItemHierarchy parms, IProgressMonitor monitor)
			throws TeamRepositoryException {

		parms.validate("getWorkItemHierarchy"); //$NON-NLS-1$

		SubMonitor progress = SubMonitor.convert(monitor, 100);
		
		// create locate change sets parms from work item hierarchy parms
		ParmsGetLcsEditorChangeSets lcsParms = new ParmsGetLcsEditorChangeSets();
		lcsParms.repo = CommonUtil.getTeamRepository(parms.repositoryUrl);
		lcsParms.maxChildWorkItemsDepth = parms.searchDepth;
		lcsParms.maxChangeSets = LocateChangeSetsUtil.UNBOUNDED;
		lcsParms.maxWorkItems = LocateChangeSetsUtil.UNBOUNDED;

		lcsParms.changeSets = new ArrayList<IChangeSetHandle>();
		if (parms.changeSets != null) {
			for (ParmsChangeSet changeSet : parms.changeSets) {
				lcsParms.changeSets.add(CommonUtil.createChangeSetHandle(
						lcsParms.repo, changeSet.changeSetItemId));
			}
		}

		lcsParms.workItems = new ArrayList<IWorkItemHandle>();
		if (parms.workItems != null) {
			for (ParmsWorkItem workItem : parms.workItems) {
				lcsParms.workItems.add((IWorkItemHandle) IWorkItem.ITEM_TYPE
						.createItemHandle(lcsParms.repo, UUID
								.valueOf(workItem.workItemId), null));
			}
		}
		
		progress.worked(5);

		// get locate change sets hierarchy
		LcsEditorChangeSetsResult result = LocateChangeSetsUtil
				.getLcsEditorChangeSets(lcsParms, progress.newChild(60));
		
		// collect all contributor and component items
		Map<UUID, IContributor> contributors = new HashMap<UUID, IContributor>();
		Map<UUID, IComponent> components = new HashMap<UUID, IComponent>();
		collectItems(lcsParms.repo, result, contributors, components, progress.newChild(30));

		// create work item hierarchy root from locate change sets result
		WorkItemHierarchyRootDTO rootNode = FilesystemWorkItemsRestDTOhierarchyFactory.eINSTANCE
				.createWorkItemHierarchyRootDTO();

		LcsEditorUnassignedWorkItemEntry uaEntry = result.getUnassignedRoot();
		if (uaEntry != null) {
			Collection<LcsEditorEntry> unassignedRoots = uaEntry.getChildren();
			if (unassignedRoots != null) {
				for (LcsEditorEntry entry : unassignedRoots) {
					if (entry instanceof LcsEditorChangeSetEntry) {
						LcsEditorChangeSetEntry csEntry = (LcsEditorChangeSetEntry) entry;
						ChangeSetDTO changeSet = createChangeSetDTO(lcsParms.repo, csEntry, contributors, components);
						rootNode.getChangeSets().add(changeSet);
					}
				}
			}
		}

		Collection<LcsEditorWorkItemEntry> rootEntries = result.getRoots();
		if (rootEntries != null) {
			for (LcsEditorWorkItemEntry wiEntry : rootEntries) {
				WorkItemHierarchyNodeDTO wiNode = createWorkItemNode(
						lcsParms.repo, wiEntry, contributors, components);
				rootNode.getChildren().add(wiNode);
			}
		}
		
		progress.worked(5);

		return rootNode;
	}

	// collect all items in fewer fetches
	private void collectItems(ITeamRepository repo,
			LcsEditorChangeSetsResult result,
			Map<UUID, IContributor> contributorsMap,
			Map<UUID, IComponent> componentsMap, IProgressMonitor monitor)
			throws TeamRepositoryException {
		
		SubMonitor progress = SubMonitor.convert(monitor, 100);

		List<IContributorHandle> contributorHandles = new ArrayList<IContributorHandle>();
		List<IComponentHandle> componentHandles = new ArrayList<IComponentHandle>();

		collectHandles(result, contributorHandles, componentHandles);

		List<IContributor> contributors = (List<IContributor>) repo
				.itemManager().fetchCompleteItems(contributorHandles,
						IItemManager.DEFAULT, progress.newChild(50));

		List<IComponent> components = (List<IComponent>) repo.itemManager()
				.fetchCompleteItems(componentHandles,
						IItemManager.DEFAULT, progress.newChild(50));

		for (IContributor contributor : contributors) {
			contributorsMap.put(contributor.getItemId(), contributor);
		}

		for (IComponent component : components) {
			componentsMap.put(component.getItemId(), component);
		}
	}

	// collect all handles from result
	private void collectHandles(LcsEditorChangeSetsResult result,
			List<IContributorHandle> contributorHandles,
			List<IComponentHandle> componentHandles)
			throws TeamRepositoryException {

		collectHandles(new ArrayList<LcsEditorEntry>(result.getRoots()),
				contributorHandles, componentHandles);
		LcsEditorUnassignedWorkItemEntry unassignedRoot = result
				.getUnassignedRoot();
		if (unassignedRoot != null) {
			collectHandles(unassignedRoot.getChildren(), contributorHandles,
					componentHandles);
		}
	}

	// recursively collect all handles from children
	private void collectHandles(Collection<LcsEditorEntry> entries,
			List<IContributorHandle> contributorHandles,
			List<IComponentHandle> componentHandles) {
		for (LcsEditorEntry entry : entries) {
			if (entry instanceof LcsEditorWorkItemEntry) {
				LcsEditorWorkItemEntry wiEntry = (LcsEditorWorkItemEntry) entry;
				collectHandles(wiEntry.getChildren(), contributorHandles,
						componentHandles);
			} else if (entry instanceof LcsEditorChangeSetEntry) {
				LcsEditorChangeSetEntry csEntry = (LcsEditorChangeSetEntry) entry;
				IChangeSet changeSet = csEntry.getChangeSet();
				contributorHandles.add(changeSet.getAuthor());
				componentHandles.add(changeSet.getComponent());
				collectHandles(csEntry.getChildren(), contributorHandles,
						componentHandles);
			}

		}
	}

	// create a work item hierarchy node from an LCS work item entry
	private WorkItemHierarchyNodeDTO createWorkItemNode(ITeamRepository repo,
			LcsEditorWorkItemEntry wiEntry, Map<UUID, IContributor> contributors, Map<UUID, IComponent> components)
			throws TeamRepositoryException {
		WorkItemHierarchyNodeDTO node = FilesystemWorkItemsRestDTOhierarchyFactory.eINSTANCE
				.createWorkItemHierarchyNodeDTO();
		WorkItemDTO workItem = createWorkItemDTO(wiEntry);
		node.setWorkItem(workItem);
		addChildren(repo, node, wiEntry, contributors, components);
		node.setRepositoryURL(repo.getRepositoryURI());
		return node;
	}

	// create a work item DTO from an LCS work item entry
	private WorkItemDTO createWorkItemDTO(LcsEditorWorkItemEntry wiEntry) {
		WorkItemDTO workItem = FilesystemWorkItemsRestDTOFactory.eINSTANCE
				.createWorkItemDTO();
		workItem.setItemId(wiEntry.getWorkItem().getItemId().getUuidValue());
		workItem.setId(wiEntry.getWorkItem().getId());
		workItem.setSummary(wiEntry.getWorkItem().getHTMLSummary()
				.getPlainText());
		return workItem;
	}

	// add children to work item hierarchy node from locate change sets work
	// item entry
	private void addChildren(ITeamRepository repo,
			WorkItemHierarchyNodeDTO parentNode,
			LcsEditorWorkItemEntry parentEntry, Map<UUID, IContributor> contributors, Map<UUID, IComponent> components)
			throws TeamRepositoryException {

		for (LcsEditorEntry entry : parentEntry.getChildren()) {

			if (entry instanceof LcsEditorWorkItemEntry) {
				LcsEditorWorkItemEntry wiEntry = (LcsEditorWorkItemEntry) entry;
				WorkItemHierarchyNodeDTO wiNode = createWorkItemNode(repo,
						wiEntry, contributors, components);
				parentNode.getChildren().add(wiNode);

			} else if (entry instanceof LcsEditorChangeSetEntry) {
				LcsEditorChangeSetEntry csEntry = (LcsEditorChangeSetEntry) entry;

				ChangeSetDTO changeSet = createChangeSetDTO(repo, csEntry, contributors, components);
				parentNode.getChangeSets().add(changeSet);
			}
		}
	}

	// create a change set DTO from an LCS change set entry
	private ChangeSetDTO createChangeSetDTO(ITeamRepository repo,
			LcsEditorChangeSetEntry csEntry, Map<UUID, IContributor> contributors, Map<UUID, IComponent> components)
			throws TeamRepositoryException {

		ChangeSetDTO changeSetDTO = FilesystemRestClientDTOcoreFactory.eINSTANCE
				.createChangeSetDTO();
		changeSetDTO.setItemId(csEntry.getChangeSet().getItemId().getUuidValue());
		changeSetDTO.setComment(csEntry.getChangeSet().getComment());
		changeSetDTO.setLastChangeDate(csEntry.getChangeSet()
				.getLastChangeDate().getTime());

		IContributorHandle contributorHandle = csEntry.getChangeSet().getAuthor();
		IContributor contributor = contributors.get(contributorHandle.getItemId());
		ContributorDTO contributorDTO = FilesystemRestClientDTOcoreFactory.eINSTANCE.createContributorDTO();
		contributorDTO.setFullName(contributor.getName());
		contributorDTO.setEmailAddress(contributor.getEmailAddress());
		contributorDTO.setUserId(contributor.getUserId());
		contributorDTO.setContributorItemId(contributor.getItemId().getUuidValue());
		changeSetDTO.setAuthor(contributorDTO);

		IComponentHandle componentHandle = csEntry.getChangeSet().getComponent();
		IComponent component = components.get(componentHandle.getItemId());
		ComponentDTO componentDTO = FilesystemRestClientDTOcoreFactory.eINSTANCE.createComponentDTO();
		componentDTO.setItemId(component.getItemId().getUuidValue());
		componentDTO.setName(component.getName());
		changeSetDTO.setComponent(componentDTO);

		return changeSetDTO;
	}
	
	private void createCsCrLink(ParmsPostChangeSetWorkItem parms,ITeamRepository sourceRepo, IWorkspace workspace, IChangeSetHandle changeSetHandle,
			IProgressMonitor monitor) throws TeamRepositoryException {
		
		ITeamRepository targetRepo = CommonUtil.getTeamRepository(parms.wiRepositoryUrl);
		IWorkItemHandle workItemHandle = (IWorkItemHandle) IWorkItem.ITEM_TYPE.createItemHandle(targetRepo, UUID.valueOf(parms.workItemItemId), NULL_STATE);
		
		IAuditableClient auditableClient = (IAuditableClient) targetRepo.getClientLibrary(IAuditableClient.class);
		IWorkItem wi = auditableClient.fetchCurrentAuditable(workItemHandle,IWorkItem.FULL_PROFILE , monitor);	

		List<String> comments = Collections.singletonList(null);
		List<String> extraInfos = Collections.singletonList(ChangeSetLinks.serializeExtraInfo(targetRepo.getRepositoryURI(), targetRepo.getId()));

		final IWorkspaceManager wm = SCMPlatform.getWorkspaceManager(sourceRepo);
		URI uri = getWiUri(targetRepo, wi); 
		
		CreateLinksResult result = wm.createLinks(ILinkConstants.CHANGESET_OSLC_CM_CHANGEREQUEST_LINKTYPE_ID, workspace, changeSetHandle, Collections.singletonList(uri), comments, extraInfos, null, true, false, monitor);
        if (result.getLinksAdded().size() == 0) {
            throw new TeamRepositoryException(Messages.CreateWorkitemOslcLink_NO_LINK_CREATED);
        }
	}
	
	private void removeCsCrLink(ParmsPostChangeSetWorkItem parms, ITeamRepository sourceRepo, IChangeSetHandle changeSetHandle,
			IProgressMonitor monitor) throws TeamRepositoryException {
		ITeamRepository wiRepo = CommonUtil.getTeamRepository(parms.wiRepositoryUrl);
		IWorkItemHandle workItemHandle = (IWorkItemHandle) IWorkItem.ITEM_TYPE.createItemHandle(wiRepo, UUID.valueOf(parms.workItemItemId), NULL_STATE);	
		IAuditableClient auditableClient = (IAuditableClient) wiRepo.getClientLibrary(IAuditableClient.class);
		IWorkItem wi = auditableClient.fetchCurrentAuditable(workItemHandle,IWorkItem.FULL_PROFILE , monitor);	
		URI wiUri = getWiUri(wiRepo, wi); 
		
		ProviderFactory providerFactory = (ProviderFactory) sourceRepo.getClientLibrary(ProviderFactory.class);
		List<ILink> links = ChangeSetLinks.findLinks(providerFactory, changeSetHandle, new String[] {ILinkConstants.CHANGESET_OSLC_CM_CHANGEREQUEST_LINKTYPE_ID }, monitor);
		List<ILink> failed = new ArrayList<ILink>();
		ILink linkToRemove = null;
		for (ILink link : links) {
			URI uri = (URI) link.getTargetRef().createURI();				
			if (uri.toString().equals(wiUri.toString())) {		
				linkToRemove = link;
				try {				
					OslcLinkUtil.removeOslcCMBacklink(sourceRepo, changeSetHandle, wiRepo, uri, monitor);
					// back link has either been removed or could not be found
				} catch (TeamRepositoryException e) {
					// back link could not be removed or change request could not even be accessed
					LoggingHelper.log(uri.toString(), e);
					failed.add(link);
				}
				//WorkItem to remove is found exit 'for' loop
				break;
			}
		}

		if (linkToRemove != null) {
			OslcLinkUtil.deleteScmOslcLinks(sourceRepo, Collections.singletonList(linkToRemove), monitor);
		}
		else {
			throw new TeamRepositoryException(NLS.bind(Messages.RemoveWorkitemOslcLink,changeSetHandle.getItemId().getUuidValue(), wi.getId()));
		}
	}
	
	private URI getWiUri(ITeamRepository repo, IWorkItem wi) throws TeamRepositoryException{
		URI uri = null;
		try {
			 uri = new URI(Location.namedLocation(wi, repo.getRepositoryURI()).toString());			 
		} catch (URISyntaxException e) {
            throw new TeamRepositoryException(e);
		}
		return uri;
	}
	
	/* (non-Javadoc)
	 * @see com.ibm.team.filesystem.client.workitems.IFileSystemWorkItemRestClient#postRefreshItems(com.ibm.team.filesystem.client.workitems.rest.parameters.ParmsRefreshItems, org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void postRefreshWorkItems(ParmsRefreshItems parms,
			IProgressMonitor monitor) throws TeamRepositoryException {
		ITeamRepository repo = CommonUtil.getTeamRepository(parms.repositoryUrl);
		
		List<IWorkItemHandle> workItems = new ArrayList<IWorkItemHandle>(parms.workItemIds.length);
		for (String wid : parms.workItemIds) {
			IWorkItemHandle workItemHandle = (IWorkItemHandle) IWorkItem.ITEM_TYPE.createItemHandle(repo, UUID.valueOf(wid), NULL_STATE);
			workItems.add(workItemHandle);
		}
		
		IAuditableClient auditableClient = (IAuditableClient) repo.getClientLibrary(IAuditableClient.class);
		auditableClient.fetchCurrentAuditables(workItems, IWorkItem.FULL_PROFILE, monitor);
		
	}
}
