/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client.workitems.internal;

import com.ibm.team.filesystem.client.IOperationFactory;
import com.ibm.team.filesystem.client.operations.DeliverDilemmaHandler;
import com.ibm.team.filesystem.client.operations.IFileSystemOperation;
import com.ibm.team.filesystem.client.operations.OperationOptions;
import com.ibm.team.filesystem.client.workitems.IWorkItemOperationFactory;
import com.ibm.team.filesystem.client.workitems.internal.operations.CheckinAndDeliverOperation;
import com.ibm.team.filesystem.client.workitems.internal.operations.DeliverAndFixOperation;
import com.ibm.team.filesystem.client.workitems.internal.operations.SubmitForReviewOperation;
import com.ibm.team.filesystem.client.workitems.operations.DeliverAndFixDilemmaHandler;
import com.ibm.team.filesystem.client.workitems.operations.ICheckinAndDeliverOperation;
import com.ibm.team.filesystem.client.workitems.operations.IDeliverAndFixOperation;
import com.ibm.team.filesystem.client.workitems.operations.ISubmitForReviewOperation;
import com.ibm.team.filesystem.client.workitems.operations.SubmitForReviewDilemmaHandler;

public class WorkItemOperationFactory implements IWorkItemOperationFactory {

	private static IWorkItemOperationFactory operationFactory;

	public static IWorkItemOperationFactory getInstance() {
		if (operationFactory == null) {
			operationFactory = new WorkItemOperationFactory();
		}
		return operationFactory;
	}

	private WorkItemOperationFactory() { }; // singleton
    
    private <T extends IFileSystemOperation> T configure(T object) {
    	OperationOptions options = IOperationFactory.instance.getOperationOptions();
        options.configureOperation(object);
        return object;
    }

	public IDeliverAndFixOperation getDeliverAndResolveOperation(DeliverAndFixDilemmaHandler dilemmaHandler) {
		return configure(new DeliverAndFixOperation(dilemmaHandler));
	}

	public ISubmitForReviewOperation getSubmitForReviewOperation(SubmitForReviewDilemmaHandler dilemmaHandler) {
		return configure(new SubmitForReviewOperation(dilemmaHandler));
	}

	public ICheckinAndDeliverOperation getCheckinAndDeliverOperation(
			DeliverDilemmaHandler dilemmaHandler) {
		return configure(new CheckinAndDeliverOperation(dilemmaHandler));
	}
}
