/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2010. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client.workitems.internal.advisors;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.scm.client.internal.ClientProviderFactory;
import com.ibm.team.scm.client.internal.ProcessClientProvider;
import com.ibm.team.scm.common.process.DeliverOperationData;

/**
 * Operation advisor which validates that all change sets have an associated
 * work item and/or comment.
 */
public class RequireWorkItemAdvisor implements IOperationAdvisor {

	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		Object opData = operation.getOperationData();
		if (!(opData instanceof DeliverOperationData)) {
			return;
		}
		DeliverOperationData data = (DeliverOperationData)opData;
		ITeamRepository repo = (ITeamRepository)data.getSourceWorkspace().getOrigin();
		ITeamRepository destRepo = (ITeamRepository)data.getDestWorkspace().getOrigin();
		ClientProviderFactory provider = new ClientProviderFactory(repo);
		if (destRepo != null && !destRepo.getId().equals(repo.getId())) {
			// Use the process from the target repository
			provider.setProcessProvider(new ProcessClientProvider(destRepo));
		}
		
		com.ibm.team.filesystem.common.workitems.internal.process.RequireWorkItemAdvisor implementation = 
			new com.ibm.team.filesystem.common.workitems.internal.process.RequireWorkItemAdvisor(provider);
		implementation.run(operation, advisorConfiguration, collector, monitor);
	}
}
