/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client.workitems.operations;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;

import com.ibm.team.filesystem.client.IWorkItemsLinksRemover;
import com.ibm.team.filesystem.client.workitems.IFileSystemWorkItemManager;
import com.ibm.team.links.common.ILink;
import com.ibm.team.links.common.ILinkHandle;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.scm.client.IWorkspaceConnection;
import com.ibm.team.scm.common.IChangeSetHandle;
import com.ibm.team.scm.common.dto.IChangeSetLinkSummary;
import com.ibm.team.scm.common.links.ChangeSetLinks;
import com.ibm.team.scm.common.links.ILinkConstants;
import com.ibm.team.scm.common.providers.ProviderFactory;

public class DefaultWorkItemsLinksRemover implements IWorkItemsLinksRemover {

	@Override
	public void removeWorkItemLinks(IWorkspaceConnection connection,
			List<IChangeSetHandle> changeSetHandles, IProgressMonitor monitor) throws TeamRepositoryException {
		ITeamRepository repo = connection.teamRepository();

		IFileSystemWorkItemManager manager = (IFileSystemWorkItemManager) repo.getClientLibrary(IFileSystemWorkItemManager.class);
		ProviderFactory providerFactory = (ProviderFactory) repo.getClientLibrary(ProviderFactory.class);

		List<ILinkHandle> linkHandles = new ArrayList<ILinkHandle>();

		List<IChangeSetLinkSummary> changeSetLinkSummary = providerFactory.getScmProvider().getChangeSetLinkSummary(changeSetHandles, SubMonitor.convert(monitor, changeSetHandles.size()));
		for (IChangeSetLinkSummary summary : changeSetLinkSummary) {
			linkHandles.addAll(summary.getLinks());
		}
		@SuppressWarnings("unchecked")
		List<ILink> links = (List<ILink>) repo.itemManager().fetchCompleteItems(linkHandles, IItemManager.DEFAULT, SubMonitor.convert(monitor, linkHandles.size()));

		links = ChangeSetLinks.filter(links, ILinkConstants.CHANGESET_WORKITEM_LINKTYPE_ID);
		int size = links.size();
		if (size > 0) {
			manager.deleteLinks(links, SubMonitor.convert(monitor, size));
		}
	}

}
