/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client.workitems.operations;

import java.util.List;

import com.ibm.team.filesystem.client.ILocalChange;
import com.ibm.team.filesystem.client.operations.ICheckinOptions;
import com.ibm.team.filesystem.client.operations.IFileSystemOperation;
import com.ibm.team.filesystem.client.operations.IRefreshingOperation;
import com.ibm.team.scm.client.IWorkspaceConnection;
import com.ibm.team.scm.common.IComponentHandle;
import com.ibm.team.scm.common.dto.IChangeHistorySyncReport;
import com.ibm.team.workitem.common.model.IWorkItemHandle;

/**
 * Operation that checks in a collection of local changes.
 * associates a work item with the change-set
 * adds a comment to the change-set
 * delivers the change-set
 */
public interface ICheckinAndDeliverOperation extends IFileSystemOperation, ICheckinOptions, IRefreshingOperation {
	/**
	 * Identify the work items the change sets are to be linked to and the comment for the 
	 * change set.
	 * @param workItem
	 * @param changeSetComment
	 */
	public void changeSetLinkage(IWorkItemHandle workItem, String changeSetComment);
	
	/**
	 * The changes specified will be checked in and delivered between 
	 * the Source workspace and the target workspace.
	 * @param source
	 * @param target
	 * @param report
	 * @param component
	 * @param localChanges
	 */
	public void add(IWorkspaceConnection source, IWorkspaceConnection target, IChangeHistorySyncReport report, 
			IComponentHandle component, List<ILocalChange> localChanges);

}
