/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2014. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client.workitems.operations;

import com.ibm.team.filesystem.client.operations.DeliverDilemmaHandler;
import com.ibm.team.filesystem.client.operations.ICheckinOptions;
import com.ibm.team.filesystem.client.operations.IFileSystemOperation;
import com.ibm.team.filesystem.client.operations.IRefreshingOperation;

/**
 * A deliver and resolve operation that can be used to deliver change sets in a 
 * workspace to a stream or another workspace (if permissions allow this) and 
 * then resolve associated work items. The deliver operation supports batching 
 * of deliver requests and delivering all changes from a source to a target. You
 * can add multiple deliver parameters to the operation and the operation will 
 * take care of batching and performing the least amount of round-trips 
 * possible. One work item can be given to associate with the change set that 
 * will be resolved with delivery.
 * <p>
 * This operation is intended to be configured and then run once.
 * It is not expected to be re-configured and run again.
 * </p>
 * <p>
 * This operation also allows the client to provide a handler which is
 * responsible for making decisions about the behavior of the operation. 
 * See {@link DeliverDilemmaHandler} for details on how to configure the
 * operation.
 * </p>
 * @NoImplement This interface is not intended to be implemented by clients.
 */
public interface IDeliverAndFixOperation extends IFileSystemOperation, IRefreshingOperation, ICheckinOptions {

    /**
     * Adds a deliver and resolve request to the operation to deliver all the
     * change sets and associate the work item, if given, then resolve all
     * associated work items.
     * 
     * @param reviewParameter parameter object that holds the change sets and
     *      data associated with the operation, never <code>null</code>, 
     *      See {@link DeliverAndFixParameter}
     */
    public void deliverAndFix(DeliverAndFixParameter reviewParameter);
    
    /**
     * When called with <code>true</code>, it will release the locks after completing the operation.
     * 
     * @param value
     */
    public void setAutoReleaseLocks(boolean value);
}
