/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client.workitems.operations;

import com.ibm.team.filesystem.client.operations.ICheckinOptions;
import com.ibm.team.filesystem.client.operations.IFileSystemOperation;
import com.ibm.team.filesystem.client.operations.IRefreshingOperation;
import com.ibm.team.filesystem.client.operations.SuspendDilemmaHandler;

/**
 * A submit change set for review operation. This operation can associate work items
 * to a group of change sets and add approvals to every associated work item. 
 * The operation supports submitting approvals for change sets with different source
 * workspaces. Multiple requests can be submitted to the operation before execution.
 * <p>
 * The operation is for one time use. Once it has been run, another {@link ISubmitForReviewOperation}
 * should be created for execution.
 * <p>
 * This operation requires a {@link SuspendDilemmaHandler} to handle decisions if
 * the change sets are to be suspended.
 */
public interface ISubmitForReviewOperation extends IFileSystemOperation, IRefreshingOperation, ICheckinOptions {

	/**
	 * Adds a submit for review request for a group of change sets. The review
	 * parameter supplies the change sets and related work items. The work items
	 * may contain work items that are not linked to any of the change sets. They
	 * will be linked to the change sets during the operation.
	 * @param reviewParameter
	 */
	public void submitForReview(SubmitForReviewParameter reviewParameter);
	
}
