/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.client.workitems.rest.parameters;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.filesystem.client.internal.rest.CommonUtil;
import com.ibm.team.filesystem.client.internal.rest.IValidatingParameterWrapper;
import com.ibm.team.filesystem.client.internal.rest.ParmValidation;
import com.ibm.team.filesystem.client.workitems.internal.WorkItemUtils;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.client.TeamPlatform;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.workitem.common.model.IWorkItemHandle;

public class ParmsWorkItem implements IValidatingParameterWrapper {

    /**
     * Work item id. Required.
     */
    public String workItemId;
    
    /**
     * The repository that contains the work item. Required.
     */
    public String repositoryUri;
    
    public ParmsWorkItem() {
    	this(null, null);
    }
    
    public ParmsWorkItem(String workItemId, String repositoryUri) {
        this.workItemId = workItemId;
        this.repositoryUri = repositoryUri;
    }

    public void validate(String requestName, Object... parameterName) {
        ParmValidation.requiredValue(repositoryUri, requestName, parameterName, "repositoryUri");  //$NON-NLS-1$
        ParmValidation.requiredValue(workItemId, requestName, parameterName, "workItemId");  //$NON-NLS-1$
    }
    
    /**
     * Creates a work item handle. 
     * @param monitor
     * @return the work item handle
     * @throws TeamRepositoryException
     */
    public IWorkItemHandle getWorkItem(IProgressMonitor monitor) throws TeamRepositoryException {
        ITeamRepository repo = CommonUtil.getTeamRepository(repositoryUri);
		return WorkItemUtils.getWorkItemHandle(repo, workItemId, monitor);
    }
}
