/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
package com.ibm.team.filesystem.client.workitems.rest.parameters;

import com.ibm.team.filesystem.client.internal.rest.ParmValidation;
import com.ibm.team.filesystem.client.rest.IFilesystemRestClient;
import com.ibm.team.repository.common.transport.IParameterWrapper;

/**
 * Dilemma handling for unreachable work items during various operations.
 */
public class ParmsWorkItemDilemmaHandler implements IParameterWrapper {
	/**
	 * Specifies what to do if there are unreachable work items.
	 * Default is "fail"
	 * <br>
	 * "continue" - Continue with the reachable work items.
	 * "fail" - Fail the operation.
	 * "cancel" - Cancel the operation.
	 */
	public String unreachableWorkItemInstruction;
	
    /**
     * Validate that the structure has all required information.
     * 
     * @param requestName The request being validated, used when reporting
     * illegal arguments encountered during validation.
     */
    public void validate(String requestName, Object... parameterName) {
    	if (unreachableWorkItemInstruction == null) {
    		unreachableWorkItemInstruction = IFilesystemRestClient.FAIL;
    	} else {
            ParmValidation.inEnum(unreachableWorkItemInstruction, requestName,
                    new String[] {
                    IFilesystemRestClient.CONTINUE,
                    IFilesystemRestClient.CANCEL,
                    IFilesystemRestClient.FAIL},
                    parameterName, "unreachableWorkItemInstruction"); //$NON-NLS-1$
    	}
    }
}
